@@ -4,8 +4,8 @@
 // Author:      Simon Flack
 // Modified by:
 // Created:     28/08/2002
-// RCS-ID:      $Id: docview.h,v 1.22 2006/09/24 17:15:59 mbarbon Exp $
-// Copyright:   (c) 2002-2004, 2005-2006 Simon Flack
+// RCS-ID:      $Id: docview.h,v 1.24 2007/03/21 22:15:03 mbarbon Exp $
+// Copyright:   (c) 2002-2004, 2005-2007 Simon Flack
 // Licence:     This program is free software; you can redistribute it and/or
 //              modify it under the same terms as Perl itself
 /////////////////////////////////////////////////////////////////////////////
@@ -276,6 +276,144 @@ wxWindow *wxPliDocument::GetDocumentWindow() const
 WXPLI_IMPLEMENT_DYNAMIC_CLASS( wxPliDocument, wxDocument );
 
 
+// --- Wx::View -------------------------------------------------
+
+
+class wxPliView : public wxView
+{
+    WXPLI_DECLARE_DYNAMIC_CLASS( wxPliView );
+    WXPLI_DECLARE_V_CBACK();
+public:
+    wxPliView( const char* package )
+       : wxView(),
+         m_callback( "Wx::View" )
+    {
+       m_callback.SetSelf( wxPli_make_object( this, package ), true);
+    }
+    ~wxPliView();
+
+    void OnActivateView( bool, wxView*, wxView* );
+    void OnPrint( wxDC*, wxObject* );
+    void OnUpdate( wxView* sender, wxObject* hint=(wxObject*) NULL );
+
+    DEC_V_CBACK_VOID__VOID( OnClosingDocument );
+    DEC_V_CBACK_VOID__VOID( OnChangeFilename );
+    bool OnCreate( wxDocument*, long );
+    bool Close( bool deleteWindow = true );
+    bool OnClose( bool );
+
+    // bool ProcessEvent(wxEvent&);
+    void Activate( bool );
+
+    virtual void OnDraw(wxDC* dc);
+
+#if wxUSE_PRINTING_ARCHITECTURE
+    wxPrintout* OnCreatePrintout();
+#endif
+};
+
+wxPliView::~wxPliView() {}
+
+void wxPliView::OnDraw( wxDC* dc )
+{
+}
+
+void wxPliView::OnActivateView( bool activate, wxView* activeView,
+                                wxView* deactiveView)
+{
+    dTHX;
+    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback,
+                                           "OnActivateView" ) )
+    {
+      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
+                                         "bOO", activate, activeView,
+                                         deactiveView);
+      return;
+    }
+  wxView::OnActivateView( activate, activeView, deactiveView);
+}
+
+
+void wxPliView::OnPrint( wxDC* dc, wxObject* info)
+{
+    dTHX;
+    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "OnPrint" ) )
+    {
+      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
+                                         "OO", dc, info );
+      return;
+    }
+  wxView::OnPrint( dc, info);
+}
+
+void wxPliView::OnUpdate( wxView* sender, wxObject* hint )
+{
+    dTHX;
+    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "OnUpdate" ) )
+    {
+      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
+                                         "OO", sender, hint );
+      return;
+    }
+  wxView::OnUpdate( sender, hint );
+}
+
+DEF_V_CBACK_VOID__VOID( wxPliView, wxView, OnClosingDocument );
+DEF_V_CBACK_VOID__VOID( wxPliView, wxView, OnChangeFilename );
+
+bool wxPliView::OnCreate( wxDocument* doc, long flags )
+{
+    dTHX;
+    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "OnCreate" ) )
+    {
+      SV* ret = wxPliVirtualCallback_CallCallback( aTHX_ &m_callback,
+                                                   G_SCALAR,"Ol", doc, flags );
+      bool val = SvTRUE( ret );
+      SvREFCNT_dec( ret );
+      return val;
+    }
+  return wxView::OnCreate( doc, flags );
+}
+
+DEF_V_CBACK_BOOL__BOOL( wxPliView, wxView, Close );
+DEF_V_CBACK_BOOL__BOOL( wxPliView, wxView, OnClose );
+
+void wxPliView::Activate( bool activate )
+{
+    dTHX;
+    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "Activate" ) )
+    {
+      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
+                                         "b", activate );
+      return;
+    }
+  wxView::Activate( activate );
+}
+
+
+#if wxUSE_PRINTING_ARCHITECTURE
+wxPrintout* wxPliView::OnCreatePrintout()
+{
+    dTHX;
+    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback,
+                                           "OnCreatePrintout" ) )
+    {
+      SV* ret = wxPliVirtualCallback_CallCallback( aTHX_ &m_callback,
+                                                   G_SCALAR, NULL);
+      wxPrintout* retval =
+        (wxPrintout*)wxPli_sv_2_object( aTHX_ ret, "Wx::Printout" );
+      SvREFCNT_dec( ret );
+      return retval;
+    }
+  return wxView::OnCreatePrintout( );
+}
+#endif
+
+WXPLI_IMPLEMENT_DYNAMIC_CLASS( wxPliView, wxView );
+
+
+
+
 // --- Wx::DocTemplate -------------------------------------------------
 
 
@@ -296,14 +434,38 @@ public:
                         viewTypeName, docClassInfo, viewClassInfo, flags ),
          m_callback( "Wx::DocTemplate" ),
          m_docClassName( docClassName ),
-         m_viewClassName( viewClassName )
+         m_viewClassName( viewClassName ),
+         m_plDocClassInfo( NULL ),
+         m_plViewClassInfo( NULL )
     {
-        m_hasDocClassInfo = docClassInfo != 0;
-        m_hasViewClassInfo = viewClassInfo != 0;
+        m_hasDocClassInfo = docClassInfo != 0 || !docClassName.empty();
+        m_hasViewClassInfo = viewClassInfo != 0 || !viewClassName.empty();
         m_callback.SetSelf( wxPli_make_object( this, package ), true);
+        if( !docClassName.empty() )
+        {
+            m_plDocClassInfo = new wxClassInfo( docClassName,
+                                            &wxDocument::ms_classInfo, NULL,
+                                            sizeof(wxPliDocument),
+                                            &fake_constructor
+                                            );
+            m_docClassInfo = m_plDocClassInfo;
+        }
+        if( !viewClassName.empty() )
+        {
+            m_plViewClassInfo = new wxClassInfo( viewClassName,
+                                             &wxView::ms_classInfo, NULL,
+                                             sizeof(wxPliView),
+                                             &fake_constructor
+                                             );
+            m_viewClassInfo = m_plViewClassInfo;
+        }
     }
 
-    ~wxPliDocTemplate() {}
+    ~wxPliDocTemplate()
+    {
+        delete m_plViewClassInfo;
+        delete m_plDocClassInfo;
+    }
 
     wxDocument *CreateDocument( const wxString& path, long flags = 0);
     wxView *CreateView( wxDocument*, long );
@@ -312,15 +474,32 @@ public:
     DEC_V_CBACK_WXSTRING__VOID_const( GetDocumentName );
 
 private:
+    static wxString m_className;
+    static wxObject* fake_constructor();
     static SV* CallConstructor( const wxString& className );
 private:
     wxString m_docClassName,
              m_viewClassName;
+    wxClassInfo *m_plDocClassInfo,
+                *m_plViewClassInfo;
     bool m_hasDocClassInfo, m_hasViewClassInfo;
 
     DEC_V_CBACK_BOOL__WXSTRING( FileMatchesTemplate );
 };
 
+wxString wxPliDocTemplate::m_className;
+
+wxObject* wxPliDocTemplate::fake_constructor()
+{
+    dTHX;
+
+    SV* obj = CallConstructor( m_className );
+    wxObject* doc = (wxDocument*)wxPli_sv_2_object( aTHX_ obj, "Wx::Object" );
+    SvREFCNT_dec( obj );
+
+    return doc;
+}
+
 SV* wxPliDocTemplate::CallConstructor( const wxString& className )
 {
     dTHX;
@@ -375,11 +554,9 @@ wxDocument *wxPliDocTemplate::CreateDocument( const wxString& path,
     }
     else
     {
+        m_className = m_docClassName;
         if( m_hasDocClassInfo )
             return wxDocTemplate::CreateDocument( path, flags );
-        SV* obj = CallConstructor( m_docClassName );
-        doc = (wxDocument*)wxPli_sv_2_object( aTHX_ obj, "Wx::Document" );
-        SvREFCNT_dec( obj );
     }
 
     return doc;
@@ -402,11 +579,9 @@ wxView *wxPliDocTemplate::CreateView( wxDocument* doc, long flags )
     }
     else
     {
+        m_className = m_viewClassName;
         if( m_hasViewClassInfo )
             return wxDocTemplate::CreateView( doc, flags );
-        SV* obj = CallConstructor( m_viewClassName );
-        view = (wxView*)wxPli_sv_2_object( aTHX_ obj, "Wx::View" );
-        SvREFCNT_dec( obj );
     }
 
     return view;
@@ -918,144 +1093,6 @@ WXPLI_IMPLEMENT_DYNAMIC_CLASS( wxPliDocManager, wxDocManager );
 
 
 
-// --- Wx::View -------------------------------------------------
-
-
-class wxPliView : public wxView
-{
-    WXPLI_DECLARE_DYNAMIC_CLASS( wxPliView );
-    WXPLI_DECLARE_V_CBACK();
-public:
-    wxPliView( const char* package )
-       : wxView(),
-         m_callback( "Wx::View" )
-    {
-       m_callback.SetSelf( wxPli_make_object( this, package ), true);
-    }
-    ~wxPliView();
-
-    void OnActivateView( bool, wxView*, wxView* );
-    void OnPrint( wxDC*, wxObject* );
-    void OnUpdate( wxView* sender, wxObject* hint=(wxObject*) NULL );
-
-    DEC_V_CBACK_VOID__VOID( OnClosingDocument );
-    DEC_V_CBACK_VOID__VOID( OnChangeFilename );
-    bool OnCreate( wxDocument*, long );
-    bool Close( bool deleteWindow = true );
-    bool OnClose( bool );
-
-    // bool ProcessEvent(wxEvent&);
-    void Activate( bool );
-
-    virtual void OnDraw(wxDC* dc);
-
-#if wxUSE_PRINTING_ARCHITECTURE
-    wxPrintout* OnCreatePrintout();
-#endif
-};
-
-wxPliView::~wxPliView() {}
-
-void wxPliView::OnDraw( wxDC* dc )
-{
-}
-
-void wxPliView::OnActivateView( bool activate, wxView* activeView,
-                                wxView* deactiveView)
-{
-    dTHX;
-    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback,
-                                           "OnActivateView" ) )
-    {
-      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
-                                         "bOO", activate, activeView,
-                                         deactiveView);
-      return;
-    }
-  wxView::OnActivateView( activate, activeView, deactiveView);
-}
-
-
-void wxPliView::OnPrint( wxDC* dc, wxObject* info)
-{
-    dTHX;
-    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "OnPrint" ) )
-    {
-      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
-                                         "OO", dc, info );
-      return;
-    }
-  wxView::OnPrint( dc, info);
-}
-
-void wxPliView::OnUpdate( wxView* sender, wxObject* hint )
-{
-    dTHX;
-    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "OnUpdate" ) )
-    {
-      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
-                                         "OO", sender, hint );
-      return;
-    }
-  wxView::OnUpdate( sender, hint );
-}
-
-DEF_V_CBACK_VOID__VOID( wxPliView, wxView, OnClosingDocument );
-DEF_V_CBACK_VOID__VOID( wxPliView, wxView, OnChangeFilename );
-
-bool wxPliView::OnCreate( wxDocument* doc, long flags )
-{
-    dTHX;
-    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "OnCreate" ) )
-    {
-      SV* ret = wxPliVirtualCallback_CallCallback( aTHX_ &m_callback,
-                                                   G_SCALAR,"Ol", doc, flags );
-      bool val = SvTRUE( ret );
-      SvREFCNT_dec( ret );
-      return val;
-    }
-  return wxView::OnCreate( doc, flags );
-}
-
-DEF_V_CBACK_BOOL__BOOL( wxPliView, wxView, Close );
-DEF_V_CBACK_BOOL__BOOL( wxPliView, wxView, OnClose );
-
-void wxPliView::Activate( bool activate )
-{
-    dTHX;
-    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback, "Activate" ) )
-    {
-      wxPliVirtualCallback_CallCallback( aTHX_ &m_callback, G_SCALAR|G_DISCARD,
-                                         "b", activate );
-      return;
-    }
-  wxView::Activate( activate );
-}
-
-
-#if wxUSE_PRINTING_ARCHITECTURE
-wxPrintout* wxPliView::OnCreatePrintout()
-{
-    dTHX;
-    if( wxPliVirtualCallback_FindCallback( aTHX_ &m_callback,
-                                           "OnCreatePrintout" ) )
-    {
-      SV* ret = wxPliVirtualCallback_CallCallback( aTHX_ &m_callback,
-                                                   G_SCALAR, NULL);
-      wxPrintout* retval =
-        (wxPrintout*)wxPli_sv_2_object( aTHX_ ret, "Wx::Printout" );
-      SvREFCNT_dec( ret );
-      return retval;
-    }
-  return wxView::OnCreatePrintout( );
-}
-#endif
-
-WXPLI_IMPLEMENT_DYNAMIC_CLASS( wxPliView, wxView );
-
-
-
-
 // --- Wx::DocChildFrame -------------------------------------------------
 
 
